# Iteration und Schleifen

Bislang haben wir einige Zeit damit verbracht, uns unterschiedliche Klänge anzusehen, die du mit `play` und `sample` erzeugen kannst. Wir haben auch gelernt, wie du diese Klänge mit `sleep` in der Zeit steuern kannst.

Wahrscheinlich hast du entdeckt, dass man mit diesen Grundbausteinen schon *eine Menge* Spaß haben kann. Eine ganz neue Dimension eröffnet sich jedoch, wenn wir beginnen, die mächtigen Möglichkeiten des Codes nutzen, die Musik und die  Kompositionen zu strukturieren. In den nächsten Abschnitten untersuchen wir einige dieser mächtigen neuen Werkzeuge. Als erstes kommen die Iteration und die Schleife (engl. *loop*) dran.

## Wiederholung

Möchtest du Code, den du geschrieben hast, einige Male wiederholen? Zum Beispiel hast du vielleicht ungefähr so etwas:

```
play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25
```

Was, wenn du das drei Mal wiederholen wolltest. Naja, du könntest das einfach so lösen, dass du den Code kopierst und drei Mal hintereinander einfügst:

```
play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25

play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25

play 50
sleep 0.5
sample :elec_blup
sleep 0.5
play 62
sleep 0.25
```

Das ist natürlich eine Menge Code! Was wäre, wenn du das Sample in `:elec_plip` umändern wolltest? Du müsstest alle Stellen mit `:elec_blub` finden und ändern. Und weiter: Was, wenn du das ursprüngliche Codestück 50 oder 1000 Mal wiederholen wolltest? Das wäre noch mehr Code, und du müsstest eine Menge Zeilen bearbeiten, wenn du etwas ändern wolltest.

## Iteration

Wenn man Code wiederholt ablaufen lassen wollte, müsste das eigentlich so einfach sein zu sagen, *mach' das drei Mal*. Und so ist es auch in etwa. Erinnerst du dich an unseren alten Freund den Codeblock? Wir können damit den Anfang und das Ende des Codes markieren, den wir drei Mal wiederholen wollen. Dann benutzen wir den speziellen Ausdruck `3.times`. Wir schreiben zwar nicht *mach' das drei Mal*, wir schreiben `3.times do` - also nicht besonders schwierig. Denk' aber daran, am Ende des Codes, den du wiederholen möchtest, ein `end` einzufügen.

```
3.times do
  play 50
  sleep 0.5
  sample :elec_blup
  sleep 0.5
  play 62
  sleep 0.25
end
```

Na, ist das nicht viel eleganter als kopieren und einfügen? Wir können das gut gebrauchen, um eine Menge sich wiederholender Strukturen zu erzeugen.

```
4.times do
  play 50
  sleep 0.5
end

8.times do
  play 55, release: 0.2
  sleep 0.25
end

4.times do
  play 50
  sleep 0.5
end
```

## Iterationen ineinander verschachteln

Wir können Iterationen in andere Iterationen hineinpacken, um damit interessante Muster zu erzeugen. Zum Beispiel:

```
4.times do
  sample :drum_heavy_kick
  2.times do
    sample :elec_blip2, rate: 2
    sleep 0.25
  end
  sample :elec_snare
  4.times do
    sample :drum_tom_mid_soft
    sleep 0.125
  end
end
```

## Schleifen

Wenn du etwas sehr oft wiederholen möchtest, könnte es sein, dass du mit sehr hohen Zahlen arbeiten musst wie zum Beispiel `1000.times do`. In einem solchen Fall macht es mehr Sinn, Sonic Pi dazu zu bringen, den Code für immer zu wiederholen (zumindest, bis du den Stop-Button klickst!). Wiederholen wir den Amen Break unendlich oft:

```
loop do
  sample :loop_amen
  sleep sample_duration :loop_amen
end
```

Eine wichtige Sache bei Schleifen ist, dass sie für den Code wie schwarze Löcher sind. Wenn der Code einmal in einer Schleife läuft, kann er diese nicht mehr verlassen, bis du auf Stop klickst - er wird für immer weiterlaufen. Das bedeutet, wenn hinter deiner Schleife noch weiterer Code steht, wirst du diesen *nie* hören. Zum Beispiel wird das Becken hinter dieser Schleife nie spielen:

```
loop do
  play 50
  sleep 1
end

sample :drum_cymbal_open
```
Jetzt fang' an und strukturiere deine Code mit Wiederholungen und Schleifen!

