# Funktionen

Wenn du einmal damit angefangen hast, mehr Code zu schreiben, dann wirst du nach Wegen suchen, wie du die Dinge organisieren und strukturieren kannst, damit alles ordentlicher wird und einfacher zu verstehen ist. Funktionen sind sehr leistungsfähig, um das zu erreichen. Sie geben uns die Möglichkeit, einem Bündel von Codezeilen einen Namen zu geben. Sehen wir uns das an.

## Funktionen definieren

```
define :foo do
  play 50
  sleep 1
  play 55
  sleep 2
end
```

Hier haben wir eine neue Funktion mit dem Namen `foo` definiert. Wir machen das mit unserem alten Freund, dem do/end-Block und dem Zauberwort `define` gefolgt von dem Namen, den wir unserer Funktion geben möchten. Wir mussten sie nicht unbedingt `foo` nennen, wir hätten sie auch irgendwie anders nennen können; zum Beispiel `bar`, `baz` oder idealerweise einen für dich bedeutsamen Namen wie `main_section` (etwa: *Hauptteil*) oder `lead_riff` (etwa: *Leitfigur*).

Denk' daran, vor den Funktionsnamen einen Doppelpunkt `:` zu stellen, wenn du sie definierst.

## Funktionen aufrufen

Wenn wir unsere Funktion definiert haben, können wir sie einfach über ihren Namen aufrufen:

```
define :foo do
  play 50
  sleep 1
  play 55
  sleep 0.5
end

foo

sleep 1

2.times do
  foo
end
```

Wir können `foo` sogar in Iterations-Blocks aufrufen und überall da, wo wir auch `play` oder `sample` hätten benutzen können. Das gibt uns ein große Ausdrucksfreiheit und wir können sinnvolle Worte bilden, die wir in unseren Kompositionen verwenden.


## Funktionen werden in unterschiedlichen Abläufen erinnert

Wenn du bislang den Run-Button geklickt hast, startete Sonic Pi jedesmal aufs Neue ohne irgendwelche Vorgaben. Es berücksichtigt nichs, außer dem, was im jeweiligen Arbeitsfenster steht. Du kannst dich nicht auf irgendwelchen Code beziehen, der in einem anderen Arbeitsfenster oder einem anderen Thread steht. Funktionen jedoch ändern das. Wenn du eine Funktion definierst, dann erinnert sich Sonic Pi daran. Probieren wir das aus. Ersetzte den den gesamten Code in deinem Arbeitsfenster durch:

```
foo
```

Klick' den Run-Button und höre, wie deine Funktion spielt. Wo wurde dieser Code gespeichert? Woher weiss Sonic Pi, was es zu spielen hat? Sonic Pi hat deine Funktion einfach erinnert - also sogar nachdem du den Code aus dem Arbeitsfenster gelöscht hast, wusste Sonic Pi, was du geschrieben hattest. Dies funktioniert nur mit Funktionen, die du mit `define` (und `defonce`) definiert hast.

## Funktionen parametrisieren

Vielleicht interessiert dich, dass du deinen Funktionen beibringen kannst, Argumente zu übernehmen, genauso, wie du `rrand` einen Minimal- und einen Maximalwert übergeben kannst. Sehen wir uns das an:

```
define :my_player do |n|
  play n
end

my_player 80
sleep 0.5
my_player 90
```

Das ist jetzt nicht besonders aufregend, zeigt aber, worum es hier geht. Wir haben unsere eigene Version von `play` mit dem Namen `my_player` erschaffen, die parametrisiert ist - also Argumente akzeptiert.

Die Parameter müssen nach dem `do` stehen, welches zum `define` des do/end-Blocks gehört; sie werden von vertikalen Torpfosten (die senkrechte Strich-Taste) umgeben und durch Kommata getrennt. Du kannst beliebige Worte als Parameternamen verwenden.

Die Zauberei findet innerhalb des `define`-do/end-Blocks statt. Du kannst die Parameternamen so benutzen, als wären sie wirkliche Werte. In diesem Beispiel spiele ich den Ton `n`. Du kannst die Parameter als eine Art Versprechen ansehen, dass sie durch wirkliche Werte ersetzt werden, wenn der Code läuft. Das machst du, indem du der Funktion einen Parameter mitgibst, wenn du sie aufrufst. Ich tue das hier mit `my_player 80`, um den Ton 80 zu spielen. Innerhalb der Funktionsdefinition wird `n` nun durch 80 ersetzt, sodass `play n` sich in `play 80` verwandelt. Wenn ich die Funktion erneut mit `my_player 90` aufrufe, wird `n` durch 90 ersetzt, sodass sich `play n` in `play 90` verwandelt.

Sehen wir uns interessantere Beispiele an:

``` 
define :chord_player do |root, repeats| 
  repeats.times do
    play chord(root, :minor), release: 0.3
    sleep 0.5
  end
end

chord_player :e3, 2
sleep 0.5
chord_player :a3, 3
chord_player :g3, 4
sleep 0.5
chord_player :e3, 3

```

Hier habe ich `repeats` so benutzt, als ob es eine Zahl in der Zeile `repeats.times do` wäre. Zusätzlich habe ich `roots` so verwendet, als ob es ein Notenname im Aufruf `play` wäre.

Hier kannst du sehen, wie wir sehr aussagekräftig und leicht lesbar schreiben können, indem wir eine Menge der Logik in eine Funktion verschieben.
