# FX hinzufügen

In diesem Abschnitt sehen wir uns zwei Effekte genauer an: Hall ('reverb') und Echo ('echo'). Wir werden lernen wie diese Effekte zu benutzen sind, wie man ihre Parameter steuern kann, und wie man sie verketten kann.

Das Sonic Pi FX System benutzt sogenannte Codeblöcke. Falls du Abschnitt 5.1 des Tutorials noch nicht gelesen hast, sieh es dir doch kurz an und komm dann hierher zurück.

## Hall ('reverb') 

Um den Hall-Effekt zu benutzen, fügen wir with_fx :reverb als Spezialcode zu unserem bestehenden Codeblock hinzu, und zwar so:

```
with_fx :reverb do
  play 50
  sleep 0.5
  sample :elec_plip
  sleep 0.5
  play 62
end
```

Wenn du nun diesen Codeblock abspielst, wirst du hören, dass der Hall-Effekt hinzugefügt wurde. Hört sich gut an, nicht wahr! Mit Hall-Effekt hört sich alles immer ziemlich gut an.

Sehen wir uns jetzt an, was passiert, wenn wir den Code ausserhalb des do/end Codeblocks hinzufügen:

```
with_fx :reverb do
  play 50
  sleep 0.5
  sample :elec_plip
  sleep 0.5
  play 62
end

sleep 1
play 55
```

Wie du sicher bemerkt hast, wird nun die letzte Note (55) nicht mehr mit Hall-Effekt abgespielt. Der Grund dafür ist, dass sich die Note ausserhalb des do/end Codeblocks befindet, und deshalb unsere Hall-Effekt Anweisungen nicht auf sie zutreffen.

Wenn du Noten vor dem do/end Codeblock einfügst, verhält es sich ganz ähnlich, und sie werden ebenfalls ohne Hall-Effekt abgespielt:

```
play 55
sleep 1

with_fx :reverb do
  play 50
  sleep 0.5
  sample :elec_plip
  sleep 0.5
  play 62
end

sleep 1
play 55
```

## Echo

Es stehen noch viele weitere FX zur Auswahl. Wie wäre es zum Beispiel mit etwas Echo?

```
with_fx :echo do
  play 50
  sleep 0.5
  sample :elec_plip
  sleep 0.5
  play 62
end
```

Die Sonic Pi FX Blocks sind besonders vielfältig einsetzbar weil sie über Parameter steuerbar sind, genauso wie die 'play' und 'sample' Befehle, die wir ja schon kennen. Ein spannender Echo Parameter ist zum Beispiel 'phase': dieser Parameter stellt die Dauer des Echos in Sekunden dar. Versuchen wir, das Echo etwas langsamer zu machen:

```
with_fx :echo, phase: 0.5 do
  play 50
  sleep 0.5
  sample :elec_plip
  sleep 0.5
  play 62
end
```

Und jetzt lass uns das Echo wieder schneller machen:

```
with_fx :echo, phase: 0.125 do
  play 50
  sleep 0.5
  sample :elec_plip
  sleep 0.5
  play 62
end
```

Wie wäre es nun mit einem Echo, welches viel langsamer ausschwingt? Wir können dafür einfach den 'decay' Wert auf 8 Sekunden verlängern:

```
with_fx :echo, phase: 0.5, decay: 8 do
  play 50
  sleep 0.5
  sample :elec_plip
  sleep 0.5
  play 62
end
```

## FX verschachteln

Einer der interessantesten Aspekte der FX Blöcke ist die Möglichkeit, sie ineinander zu verschachteln. So können eine Vielzahl verschiedener FX leicht verkettet werden. Wie wäre es zum Beispiel damit, einen Codeblock zuerst mit Echo und dann mit Hall zu versehen? Das geht ganz einfach, indem die beiden Codeblöce ineinander verschachtelt werden:

```
with_fx :reverb do
  with_fx :echo, phase: 0.5, decay: 8 do
    play 50
    sleep 0.5
    sample :elec_blup
    sleep 0.5
    play 62
  end
end
```

Stell dir das so vor, als ob die erzeugten Klänge von innen nach aussen fliessen würden. Der erzeugte Klang des innersten do/end Codeblocks (zum Beispiel 'play 50') wird zuerst durch den Echo FX Block gesendet, und danach noch durch den Hall FX Block.

Es ist ganz einfach, mit vielschichtigen Verschachtelungen verrückte Ergebnisse zu erzielen. Aber sei vorsichtig, weil FX viele Rechenkapazitäten verbrauchen, besonders dann, wenn du mehrere FX zugleich benutzt. Sei also etwas sparsam mit deinen FX, wenn du mit kapazitätsarmen Geräten wie dem Raspberry Pi arbeitest.

## Weitere FX entdecken

Sonic Pi hat eine Vielzahl an FX vorinstalliert, die du ausprobieren kannst. Um herauszufinden, welche verfügbar sind, klicke auf 'FX' auf der linken Seite des Hilfe-Fensters, dort wirst du die komplette Liste verfügbarer Optionen entdecken. Hier sind einige meiner Favoriten:

• wobble
• reverb
• echo
• distortion
• slicer

Jetzt aber los, mach was verücktes und baue überall in deinem Code FX ein um unglaubliche neue Klänge zu erzeugen!
